#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h> /* _delay_ms 262.14ms/F_CPU in MHz; _delay_us 768us/F_CPU in MHz */
#include <avr/pgmspace.h>



#include "pin_macros.h"
#include "uart.h"
#include "main.h"



//150us-25ms
//150-25000us


ISR(TIMER1_CAPT_vect){
	if (IcfState==ST_CAPTURE_RISE) {
		CaptureStart=ICR1;
		CAPTURE_FALLING;
		IcfState=ST_CAPTURE_FALLING;
		}
	else
	if (IcfState==ST_CAPTURE_FALLING){
		CaptureEnd=ICR1;
		TIMSK&=~_BV(TICIE1);//   
		IcfState=ST_CAPTURE_END;
		}
}


static void init(void){

	DRIVER(LED_BOTTOM,OUT);
	DRIVER(LED_TOP,OUT);
	
	DRIVER(DATA,OUT);
	DRIVER(SHIFT,OUT);
	DRIVER(STORAGE,OUT);
	
	DRIVER(OUT_POMP,OUT);
	
	DRIVER(OUT_TRIG,OUT);

	DRIVER(PIN_KEY_PROG,PULLUP);
	DRIVER(PIN_KEY_POMP,PULLUP);
	
	//---TIMER1---
	TCNT1=0;
	//TCCR1B|= _BV(CS10);	// 
	//TCCR1B|= _BV(ICNC1);	//.      
	//TCCR1B|= _BV(ICES1);	//       
	// ICESn =0,      ,  ICESn = 1,   . 

	//   -
	//TIMSK|=_BV(TICIE1);//      - 1
	//---TIMER2---
	TCCR2=(1<<CS22)|(1<<WGM21);//CTC
	OCR2=124;

	sei();	
}

//-------------------EEPROM-------------------------
static inline unsigned char EEPROM_read(unsigned int uiAddress)
{
	while(EECR & (1<<EEWE));
	EEAR = uiAddress;
	EECR |= (1<<EERE);
	return EEDR;
}

static void EEPROM_write(unsigned int uiAddress, unsigned char ucData)
{
	if ( EEPROM_read(uiAddress)!=ucData){
		while(EECR & (1<<EEWE));
		cli();
		asm("wdr");//  
		EEAR = uiAddress;
		EEDR = ucData;
		EECR |= (1<<EEMWE);
		EECR |= (1<<EEWE);
		sei();
		}
}

static void read_data_EEPROM(void)
{
uint8_t  *a, i, adr;
uint16_t crc=0;
	a=(uint8_t *)&setup;
	for( i=0, adr=ADR_EEP ; i<sizeof(setup); i++, a++, adr++){
		*a=EEPROM_read(adr);
		if (i<sizeof(setup)-2){
			crc+=*a;
			}
		}
	if ( crc != setup.crc ) {
		eeprom_err=1;
		}
}

static void EEPROM_save_parametr_all( void )
{
uint8_t i, *p, adr;
 
	setup.crc=eeprom_err=0;
	p=(uint8_t *) &setup;
	for(i=0, adr=ADR_EEP; i<sizeof(setup) ; i++, p++, adr++){
		EEPROM_write( adr, *p );
		if (i<sizeof(setup)-2)
			setup.crc+=*p;
		}
}
//-------------END------EEPROM----------------------


#define BCD_SIZEOF 5
void itoa2(int16_t Val, uint8_t * Buf)
{
const uint16_t step[BCD_SIZEOF]={10000,1000,100,10,1};
int16_t Subtract, Value;
uint8_t i, Dig;

	Value=Val;
	for (i=0; i<BCD_SIZEOF; i++)
	{
	Subtract=step[i];
	Dig='0';
	while(Value >= Subtract)
		{
		Dig++;
		Value-=Subtract;
		}
	Buf[i]=Dig;
	}
}

//******************74hc595***********************************
static 
void data_shift(uint8_t data)
{
	for(uint8_t i=8; i ; i--){//   8 
		if (data & _BV(7) )		ON(DATA);
                    else		OFF(DATA); 
		data<<=1;
		_delay_us(10);
		ON(SHIFT);
		_delay_us(10);
		OFF(SHIFT);
	}
	OFF(DATA);
	OFF(SHIFT);
}

static void hc595_storage(void){
	OFF(STORAGE);	
	_delay_us(10);
	ON(STORAGE);
	_delay_us(10);
	OFF(STORAGE);
	OFF(DATA);
	OFF(SHIFT);
}

void out_595(uint8_t data){
	data_shift(data);
	hc595_storage();
	data_shift(0);
}

static
void get_length_pulse(void){
	if (IcfState==ST_START_CAPTURE){
		OFF(OUT_TRIG);
		ON(OUT_TRIG);
		_delay_us(10);
		OFF(OUT_TRIG);
		
		TCNT1=0;
		CAPTURE_RISE;
		TIFR=_BV(ICF1)|_BV(TOV1);
		IcfState=ST_CAPTURE_RISE;
		TIMSK|=_BV(TICIE1);//      - 1
		}

	if (IcfState==ST_CAPTURE_END){
		IcfState=ST_CAPTURE_DATA_REC;
		PulseLength=CaptureEnd-CaptureStart;
		}
	if (IcfState != ST_IDLE)
		if ( TIFR & _BV(TOV1) ) {
			PulseLength=32700;
			IcfState=ST_CAPTURE_DATA_REC;
			}
}

static 
void skan_key(void){
static uint16_t Time;
	if ( ACTIVE(PIN_KEY_POMP) || ACTIVE(PIN_KEY_PROG) ) {
		Time++;
		if (Time>=100){
			if (ACTIVE(PIN_KEY_POMP)) Key|=KEY_POMP;
			if (ACTIVE(PIN_KEY_PROG)) Key|=KEY_PROG;
			}
		}
		else{
		Time=0;
		Key=0;
		}
}

uint16_t usr_length(void){
uint32_t Length=0;
	for(uint8_t j=0; j<4; j++){
		if (j) _delay_ms(70);
		IcfState=ST_START_CAPTURE;
		while (IcfState!=ST_CAPTURE_DATA_REC){
			get_length_pulse();
			if (TIFR & _BV(TOV1)) return 32700;
			}
		Length+=PulseLength;
		}
	return (uint16_t)(Length/4);
}

static 
void key_action(void){
static uint16_t Time;
uint8_t s[BCD_SIZEOF];

	if (StateApp==ST_WORK){
		if ( Key & (KEY_PROG | KEY_POMP) ){
			if (++Time>1000) {
				StateApp=ST_ENTER_PROG;
				if ( Key==KEY_PROG ) StateApp=ST_ENTER_PROG;
				else 
				if ( Key==KEY_POMP ) StateApp=ST_ENTER_POMP;
				}
			}
			else{
			Time=0;
			}
		}
	else
	if (StateApp==ST_ENTER_PROG){
		out_595(0xFF);
		data_shift(0xFF);
		ON(LED_BOTTOM); 
		ON(LED_TOP);
		_delay_ms(700);
		out_595(0);		
		OFF(LED_BOTTOM); 
		OFF(LED_TOP);
		_delay_ms(700);

		if ( Key==0 ) {// 1  Key   ,       
			_delay_ms(500);
			StateApp=ST_PROG_BOTTOM;
			Time=0;
			}
		}
	else
	if (StateApp==ST_PROG_BOTTOM){
		Time++;
		if (Time==1) {out_595(_BV(0)); ON(LED_BOTTOM);}
		if (Time==500) {out_595(0); OFF(LED_BOTTOM);}
		if (Time==1000) Time=0;
		if (Key==KEY_PROG){
			out_595(_BV(0));
			ON(LED_BOTTOM);
			setup.PointBottom=usr_length();//  ,    .   TOV1,    
			itoa2(setup.PointBottom,s);
			uart_puts("Bottom Level - ");
			uart_putc(s[0]);uart_putc(s[1]);uart_putc(s[2]);uart_putc(s[3]);uart_putc(s[4]);
			if ( TIFR & _BV(TOV1) ) StateApp=ST_ERROR;
			else{
				out_595(_BV(0));
				_delay_ms(1500);
				out_595(0);
				OFF(LED_BOTTOM);
				OFF(LED_TOP);
				_delay_ms(500);
				Time=0;
				Key=0;
				StateApp=ST_PROG_TOP;
				}
			}
		}
	else
	if (StateApp==ST_PROG_TOP){
		Time++;
		if (Time==1) {out_595(0);data_shift(0xFF); ON(LED_TOP);}
		if (Time==500) {out_595(0); OFF(LED_TOP);}
		if (Time==1000) Time=0;
		if (Key==KEY_PROG){
			out_595(0);data_shift(0xFF); 
			ON(LED_TOP);
			setup.PointTop=usr_length();//  ,    .   TOV1,    
			itoa2(setup.PointTop,s);
			uart_puts(" Top Level - ");
			uart_putc(s[0]);uart_putc(s[1]);uart_putc(s[2]);uart_putc(s[3]);uart_putc(s[4]);
			uart_putc('\r');
			if ( (setup.PointTop > setup.PointBottom)||(TIFR & _BV(TOV1)) ) StateApp=ST_ERROR;
			else{
				_delay_ms(1500);
				out_595(0);
				OFF(LED_BOTTOM);
				OFF(LED_TOP);
				_delay_ms(1000);
				Time=0;
				StateApp=ST_PROG_END;
				}
			}
		}
	else
	if (StateApp==ST_ENTER_POMP){
		out_595(0xFF);
		data_shift(0xFF);	
		ON(LED_BOTTOM); 
		ON(LED_TOP);
		_delay_ms(300);
		out_595(0);		
		OFF(LED_BOTTOM); 
		OFF(LED_TOP);
		_delay_ms(300);

		if ( Key==0 ) {// 1  Key   ,       
			_delay_ms(500);
			StateApp=ST_PROG_POMP;
			Time=0;
			}
		}
	else
	if (StateApp==ST_PROG_POMP){
		Time++;
		if (Time==1) {out_595(_BV(1)); ON(LED_BOTTOM);}
		if (Time==300) {out_595(0); OFF(LED_BOTTOM);}
		if (Time==600) Time=0;
		if (Key==KEY_POMP){
			out_595(_BV(1));
			ON(LED_BOTTOM);
			setup.PointPomp=usr_length();//  ,    .   TOV1,    
			itoa2(setup.PointPomp,s);
			uart_puts(" Pomp Level - ");
			uart_putc(s[0]);uart_putc(s[1]);uart_putc(s[2]);uart_putc(s[3]);uart_putc(s[4]);
			uart_putc('\r');
			if ( TIFR & _BV(TOV1) ) StateApp=ST_ERROR;
			else{
				_delay_ms(1500);
				out_595(0);
				OFF(LED_BOTTOM);
				OFF(LED_TOP);
				_delay_ms(1000);
				Time=0;
				StateApp=ST_PROG_END;
				}
			}
		}	
	else
	if (StateApp==ST_PROG_END){
		out_595(0xFF); 
		data_shift(0xFF); 
		ON(LED_BOTTOM); 
		ON(LED_TOP);
		_delay_ms(700);
		out_595(0); 
		OFF(LED_BOTTOM); 
		OFF(LED_TOP);
		_delay_ms(700);
		setup.Length100prz=setup.PointBottom-setup.PointTop;
		EEPROM_save_parametr_all();
		Out595=0;
		StateApp=ST_WORK;
		}
	else
	if (StateApp==ST_ERROR){
		for (uint8_t j=0; j<4; j++){
			out_595(0x0F); 
			ON(LED_BOTTOM); 
			OFF(LED_TOP);
			_delay_ms(300);
			out_595(0xF0); 
			OFF(LED_BOTTOM); 
			ON(LED_TOP);
			_delay_ms(300);
			}
		out_595(0); 
		OFF(LED_BOTTOM); 
		OFF(LED_TOP);
		_delay_ms(700);
		read_data_EEPROM();
		StateApp=ST_WORK;
		}
}

static 
uint8_t get_prozent_length(void){//         
static uint16_t Time;
uint16_t p;	
uint8_t s[BCD_SIZEOF];
	if (++Time>1000){
		Time=0;
		IcfState=ST_START_CAPTURE;
		}
	
	get_length_pulse();
	
	if (IcfState==ST_CAPTURE_DATA_REC){
		IcfState=ST_IDLE;
		p=PulseLength;
		itoa2(( (uint32_t)p*10UL)/58,s);
			/*uart_putc(s[0]);*/uart_putc(s[1]);uart_putc(s[2]);uart_putc(s[3]);uart_putc('.');uart_putc(s[4]);	
			uart_puts("sm ");
		itoa2(p,s);
			uart_putc(' ');
			uart_putc(s[0]);uart_putc(s[1]);uart_putc(s[2]);uart_putc(s[3]);uart_putc(s[4]);
		
		if (p>= setup.PointTop-50 ) {
			OverHiLevel=0;
			}
			else {
			p=setup.PointTop;
			OverHiLevel=1;
			}
		if (p>setup.PointBottom) {
			OverLowLevel=1;
			p=setup.PointBottom;
			}
			else {
			OverLowLevel=0;
			}
		CurrentProzentLevel= ((uint32_t)(setup.PointBottom-p)*100UL+100)/setup.Length100prz;
		itoa2(CurrentProzentLevel,s);
			uart_puts("us Prz - ");
			/*uart_putc(s[0]);uart_putc(s[1]);*/uart_putc(s[2]);uart_putc(s[3]);uart_putc(s[4]);
			uart_putc('%');
			if (OverLowLevel) uart_putc('L');
			if (OverHiLevel) uart_putc('H');
			//uart_putc('\r');
		return 1;
		}
	return 0;
}

static 
uint16_t led_state(void){//    
uint8_t s[BCD_SIZEOF];	
	//LedEndPosition=1UL<<( (CurrentProzentLevel*LED_NUM+39)/100 );//  
	LedEndPosition=1UL<<( (CurrentProzentLevel*LED_NUM)/100 );
	itoa2((CurrentProzentLevel*LED_NUM)/10,s);
	Mig=s[BCD_SIZEOF-1]-'0';
	uart_puts(" Led on - ");
	/*uart_putc(s[0]);uart_putc(s[1]);uart_putc(s[2]);*/uart_putc(s[3]);uart_putc('.');uart_putc(s[4]);
	uart_puts("%\r");
	return (LedEndPosition-1);
	
}

static 
void mig_last_led_of_overlow(void){//        
static uint16_t Time;
	if (OverHiLevel){
		if (++Time<=200){
			Out595|=1UL<<(LED_NUM-1);
			}
			else Out595&=~(1UL<<(LED_NUM-1));
		if (Time>=400) Time=0;
		}
}

static 
void pwm_last_led(void){//   
static uint8_t PwmLed=0;

	PwmLed++;
	if (PwmLed==10) {
		PwmLed=0;
		if (Mig) Out595|= LedEndPosition;
		}
		
	if (Mig==PwmLed) Out595&=~LedEndPosition;
}

static 
void control_pomp_out(void){/// 
	if (PompDisable==0){
		if ( PulseLength > setup.PointPomp){
			if (++PompDrebezg==3)	PompDisable=1;
			}
			else PompDrebezg=0;
		}
	else
		{
		if ( PulseLength+200 < setup.PointPomp){
			if (++PompDrebezg==3)	PompDisable=0;
			}
			else PompDrebezg=0;
		}
	if (PompDisable) ON(OUT_POMP); else OFF(OUT_POMP);
}

static void data_eep_in_uart(void){
uint8_t s[BCD_SIZEOF];	
	itoa2(setup.PointBottom,s);
	uart_puts("Bottom - ");
	uart_putc(s[0]); uart_putc(s[1]); uart_putc(s[2]); uart_putc(s[3]); uart_putc(s[4]);
	itoa2(setup.PointTop,s);
	uart_puts("us Top - ");
	uart_putc(s[0]); uart_putc(s[1]); uart_putc(s[2]); uart_putc(s[3]); uart_putc(s[4]);
	itoa2(setup.PointPomp,s);
	uart_puts("us Pomp - ");
	uart_putc(s[0]); uart_putc(s[1]); uart_putc(s[2]); uart_putc(s[3]); uart_putc(s[4]);
	uart_puts("us\r");
}

int main(void){

	init();
	UART_init();
	read_data_EEPROM();

	uart_puts("Device_On\r");
data_eep_in_uart();	
	for (uint8_t j=1; j<9; j++){
		out_595(_BV(j)-1);
		ON(LED_BOTTOM); 
		TOGGLE(LED_TOP);
		_delay_ms(300);
		}
	data_shift(0xFF);
	_delay_ms(300);
	OFF(LED_BOTTOM); 
	OFF(LED_TOP);	
	data_shift(0);
	_delay_ms(150);
	for (uint8_t j=7; j; j--){
		out_595(_BV(j)-1);
		_delay_ms(150);
		}
	out_595(0);
	OFF(LED_BOTTOM); 
	OFF(LED_TOP);
	_delay_ms(200);
	
	if (eeprom_err) StateApp=ST_ERROR;

	while(1)
	{
	if (TIFR & (1<<OCF2)){
		TIFR=(1<<OCF2);

		if (StateApp==ST_WORK){
			//pwm_last_led();
			//mig_last_led_of_overlow();
			//data_shift(Out595);
			if (get_prozent_length()){
				Out595=led_state();
				data_shift((uint8_t)Out595);
				hc595_storage();
				if (Out595>0xFF)
					data_shift(0xFF);
					else
					data_shift(0);
				control_pomp_out();
				}
			}
		
		skan_key();
		key_action();
		}
	}
}
