

#include <avr/io.h>
#include <avr/interrupt.h>
#include <string.h>

#include "uart.h"

#if  defined(__AVR_ATmega48__)  
#define UART0_Rx_Complete   		USART_RX_vect	//  _VECTOR(18)  /* USART Rx Complete */
#define UART0_Data_Register_Empty	USART_UDRE_vect //  _VECTOR(19)  /* USART, Data Register Empty */
#define UART0_UCSRA   	UCSR0A
#define UART0_UCSRB  	UCSR0B
#define UART0_DATA     	UDR0
#define UART0_UBRR  	UBRR0
#define UART0_UCSRC  	UCSR0C

#define UART0_UDRIE    	UDRIE0//      UART.  
#define UART0_RXCIE    	RXCIE0//    . 
#define UART0_TXCIE		TXCIE0//    . 
//---------------------------------------------------------------------------------------------------
#else
//---------------------------------------------------------------------------------------------------
#warning Mega16
#define UART0_Rx_Complete   		USART_RXC_vect	//  _VECTOR(11)  /* USART Rx Complete */
#define UART0_Data_Register_Empty	USART_UDRE_vect //   /* USART, Data Register Empty */
#define UART0_UCSRA   	UCSRA
#define UART0_UCSRB  	UCSRB
#define UART0_DATA     	UDR
#define UART0_UBRR  	UBRR
#define UART0_UCSRC  	UCSRC

#define UART0_UDRIE    	UDRIE//      UART.  
#define UART0_RXCIE    	RXCIE//    . 
#define UART0_TXCIE		TXCIE//    .
#endif


//#define UART_BAUD_SELECT ((xtalCpu)/((baudRate)*16l)-1)
#define UART_BAUD_SELECT 52//((xtalCpu)/((baudRate)*16l)-1)

#define UART_Rx_BUF_MASK	(8-1)	//0b111
#define UART_Tx_BUF_MASK	(64-1)	//0b111


volatile uint8_t UART_TxHead, UART_TxTail;/*, UART_RxHead, UartReceiveOn;*/
uint8_t	//UART_Rx_buf[UART_Rx_BUF_MASK+1],
		UART_Tx_buf[UART_Tx_BUF_MASK+1];
//UARTSTR UartTxStr;
//UARTRECSTR UartRecStr;
//volatile UARTTXSTR UartTx;


//---------------------------------------------------------------
ISR(UART0_Data_Register_Empty)
{
	if ( UART_TxHead != UART_TxTail) {
		uint8_t head_tx;
		head_tx = (UART_TxTail + 1) & UART_Tx_BUF_MASK;
        UART0_DATA = UART_Tx_buf[head_tx];  // start transmission 
		UART_TxTail = head_tx;
    }else{
        // tx buffer empty, disable UDRE interrupt 
        UART0_UCSRB &= ~_BV(UART0_UDRIE);
    }
}

//---------------------------------------------------------------
/*ISR(UART0_Rx_Complete)
{
    uint8_t data, head_rx;
    data = UART0_DATA;
	head_rx=( UART_RxHead + 1) & UART_Rx_BUF_MASK;
	UART_RxHead = head_rx;
	UART_Rx_buf[head_rx]=data;
	if ( data==END_CHARSET )
		if (UART_Rx_buf[ (head_rx-(sizeof(UartRecStr)+2)) & UART_Rx_BUF_MASK ]==BEGIN_CHARSET){
			UartReceiveOn=1;
			UART0_UCSRB &= ~_BV(UART0_RXCIE);
			}
}*/


//---------------------------------------------------------------
void UART_init(void)
{
#if  defined(__AVR_ATmega48__)  
	UART0_UBRR = UART_BAUD_SELECT;// Set baud rate 
	UART0_UCSRB = (1<<RXEN0)|(1<<TXEN0)|(1<<RXCIE0);// Enable receiver and transmitter 
	UART0_UCSRC = (1<<USBS0)|(3<<UCSZ00);// Set frame format: 8data, 2stop bit (1<<URSEL)|
	//USBS-2stop bit //0b11<<UCSZ0 8 bit
#else
	UBRRL = UART_BAUD_SELECT;// Set baud rate 
//	UCSRB = (1<<RXEN)|(1<<TXEN)|(1<<RXCIE);// Enable receiver and transmitter 
	UCSRB = (1<<TXEN);// Enable transmitter 
	UCSRC = (1<<URSEL)|(1<<USBS)|(3<<UCSZ0);// Set frame format: 8data, 2stop bit 
	//USBS-2stop bit //0b11<<UCSZ0 8 bit
#endif

}


//---------------------------------------------------------------
void uart_putc(uint8_t data)
{
	uint8_t tmphead;

	tmphead  = (UART_TxHead + 1) & UART_Tx_BUF_MASK;
    
	while ( tmphead == UART_TxTail ){
		;// wait for free space in buffer
	}
	UART_Tx_buf[tmphead] = data;
	UART_TxHead = tmphead;
	// enable UDRE interrupt 
	UART0_UCSRB   |= _BV(UART0_UDRIE);
}

//---------------------------------------------------------------
void uart_puts(char *s )
{
    while (*s) 
		uart_putc(*s++);
}

//---------------------------------------------------------------
/*void uart_puts_str(void){
//':' 'K' bb bb bb bb crc '\r'
// 0   1   2  3  4  5  6   7	
	uint8_t crc=0;
	uint8_t i, *p=(uint8_t *) &UartTx;
	for(i=0; i<sizeof(UartTx); i++, p++){
		if ( i < (sizeof(UartTx)-2) )//   crc EndCh
			{
			crc^=*p;
			}
			else
			if (i==(sizeof(UartTx)-2)) 
				*p=crc ;
		uart_putc(*p);
		}
}
*/
//---------------------------------------------------------------
/*uint8_t uart_buf_parsing(void){
//':' 'K' bb bb bb bb bb bb bb bb crc '\r'
// 0   1   2  3  4  5  6  7  8  9  10  11
uint8_t Rec=0;
	if (UartReceiveOn){
		UartReceiveOn=0;
		uint8_t Buf[11];
		uint8_t crc=0;
		uint8_t RxHead=UART_RxHead;
		for (uint8_t i=0; i<sizeof(Buf); i++){
				Buf[i]=UART_Rx_buf[ (RxHead-(sizeof(Buf)-i)) & UART_Rx_BUF_MASK ];
				if (i<sizeof(Buf)-1){//  crc
					crc^=Buf[i];
					}
				}
		
		if (Buf[sizeof(Buf)-1]==crc) 
			{
			memcpy(&UartRecStr, &Buf[1], sizeof(Buf)-2);//&Buf[1]- ":" ;  sizeof(Buf)-2   crc  '\r'
			//uart_putc(WR_OK+WriteBusy);
			//WriteBusy=1;
			Rec=1;
			}
			else{
			uart_putc(SEND_TEXT);
			uart_putc(SEND_TEXT_2);
			uart_puts("Err_Crc");
			}
		UART0_UCSRB |= _BV(UART0_RXCIE);//    .
		}
	return Rec;
}*/






















//ISR(__vector_default)
//{
// user code here
//}
//USART_TXC_vect //USART, Tx Complete

/*

#define USART_TXC_vect			_VECTOR(13) // USART, Tx Complete 
#define USART_UDRE_vect			_VECTOR(12) // USART Data Register Empty 
#define USART_RXC_vect			_VECTOR(11) // USART, Rx Complete

----UCSRA----
RXC -   .    1    
     (  UDR).  
    .   RXCIE (RXCIErt)  
 UCSRB (UCSR/jB) ,      
     

  -   .    1     
       ,    
 UDR     .   TXCIE () 
 UCSRB (UCSR/?B) ,      
   .      
     ,  
  . 1 

UDRE -    .     1 
    (      
UDR    ).   , 
       .   UDRIE 
(UDRIErt)  UCSRB (UCSR/iB) ,   
    .   , 
     

FE -   .    1   
 , . .   -    
0.     -,  1 

DOR -  .    1,     
 -       
 ,     (  ).   
         
   

UPE -    .    1,   , 
   ,    . 
         0 

U2X -   .      1,   
       16  
8,         
 .         
       

MPCM -   .      1,  
   ,  . , 
   ,  


*/
